package binomialrvls;

import java.awt.*;
import java.awt.Component;
import java.text.*;
import RVLS.*;
public class normDist extends java.awt.Canvas
{

	/**
	 * class constructor, create basic normal distribution contour
	 */
	public normDist()
	{
			initialize(9,-4.0,4.0,0.0,1.0,-4,4,true,"Normal Distribution", Color.blue);
	}

	/**
	 *set the mean (default = 0)
	 */
	public void setMean(double mean) {

		this.mean = mean;
		paramChange();
	}

	public double getMean() {
		return mean;
	}

	/**
	 *sd default = 1
	 */
	public void setSd(double sd) {

		if(sd>0) {
			this.sd = sd;
			MultConst = 1/(Math.sqrt(2*Math.PI)*sd);
			paramChange();
		}
	}

	public double getSd() {
		return sd;
	}

	/**
	 * Whether to shade in the specified area
	 */
	public void setShade(boolean shade) {

		this.shade = shade;
		if (autoRepaint) repaint();

	}

	public boolean isShade() {
		return shade;
	}

	/**
	 * if shadeBetween is true, shade between lowCutoff and upCutoff
	 * otherwise shade the tails
	 */
	public void setShadeBetween(boolean _between) {
			shadeBetween=_between;
			if (autoRepaint) repaint();
	}

	public boolean isShadeBetween() {
		return shadeBetween;
	}

	/**
	 *set up to draw Y axis
	 */
	public void setYaxis(boolean yaxis) {

		this.yaxis=yaxis;
		if (autoRepaint) repaint();
	}

	public boolean isYaxis() {
		return yaxis;
	}

	/**
	 *set up the title for this normal distribution
	 */
	public void setTitle(String title) {
		this.title = title;
		if (autoRepaint) repaint();
	}

	public String getTitle() {
		return title;
	}

	/**
	 * upperlimit on x axis, but not the upCutoff
	 */
	public void setUpLimit(double upLimit) {
		this.upLimit = upLimit;
		range = upLimit-lowLimit;
		if (autoRepaint) repaint();
	}

	public double getUpLimit() {
		return upLimit;
	}

	/**
	 * set the lower limit on x axis
	 */
	public void setLowLimit(double lowLimit) {
		this.lowLimit = lowLimit;
		range = upLimit-lowLimit;
		firstTick=lowLimit;
		if (autoRepaint) repaint();
	}

	public double getLowLimit() {
		return lowLimit;
	}

	/**
	 * set the cut off points on x axis
	 */
	public void setUpCutoff(double upCutoff) {
		this.upCutoff = upCutoff;
		if (autoRepaint) repaint();
	}

	public double getUpCutoff() {
		return upCutoff;
	}

	public void setLowCutoff(double lowCutoff) {
		this.lowCutoff = lowCutoff;
		if (autoRepaint) repaint();
	}

	public double getLowCutoff() {
		return lowCutoff;
	}

	/**
	 * set the color filling color for shaded area
	 */
	public void setFillColor(Color fillColor) {
		this.fillColor=fillColor;
	}


	public Color getFillColor() {
		return fillColor;
	}

	/**
	 * lowLimit: lower x axis limit
	 * upLimit:  upper x axis limit
	 * mumTicks: number of ticks on x axis
	 */
	public void setLimits(double lowLimit, double upLimit, int numTicks) {
			this.lowLimit=lowLimit;
			this.upLimit= upLimit;
			range = upLimit-lowLimit;
			this.numTicks=numTicks;
			firstTick=lowLimit;
			if (autoRepaint) repaint();
		}

	/**
	 * shades the specified area
	 */
	public void setShadedArea(double 	lowCutoff, double upCutoff) {
		this.lowCutoff=lowCutoff;
		this.upCutoff=upCutoff;
		shade=true;
		if (autoRepaint) repaint();
	}

	/**
	 *if autorepaint is true then changing a parameter will cause a repaint.
	 */
	public void setAutoRepaint(boolean autoRepaint) {
		this.autoRepaint=autoRepaint;
	}

	public boolean isAutoRepaint() {
		return autoRepaint;
	}


	/**
	 * set up all initialization parameters
	 */
	public void initialize(int _numTicks, double _LL, double _UL, double _mean, double _sd, double _LC, double _UC, boolean _between, String _title, Color _fillColor){

		ndx=0;
		yaxis=false;
		shade = false;
		title = _title;
		fillColor = _fillColor;
		ndec=0;
		numTicks=_numTicks-1;
		pinc = 1;

		sd=_sd;
		mean=_mean;
		lowLimit=_LL;
		firstTick=lowLimit;
		upLimit=_UL;
		lowCutoff=_LC;
		upCutoff=_UC;
		shadeBetween=_between;
		range = upLimit-lowLimit;
		w=w-2*XOFFSET;
		MultConst = 1/(Math.sqrt(2*Math.PI)*sd);
		Ymax=MultConst;

		font = new Font("TimesRoman",Font.PLAIN,10);

	}

	/**
	 * paint X axis and Y axis(if set up), and then call drawDist(g) to draw normal distribution figure
	 */
    public void paint(Graphics g) {
			Dimension dim = getSize();
			g.setColor(getBackground());
			g.fillRect(0,0,dim.width,dim.height);
			g.setColor(g.getColor());

			int adj,y;
			String num;
			y=0;
			w=getSize().width-2*XOFFSET;
			h=getSize().height-BOTTOM;
			hh=h+HADJ; //HADJ = 35 initialed in declear part
			if (setYmax==false) Ymax=MultConst;
			sp2x = range/w;
			sx2p=1/sp2x;
			sp2y= Ymax/h;
			sy2p=1/sp2y;

			g.setColor(Color.black);
			g.setFont(font);

			g.drawLine(XOFFSET,hh,XOFFSET+w,hh );
			FontMetrics met=g.getFontMetrics(font);
			int fh = met.getAscent();
			if (autoSetLimits)
				step = (upLimit-lowLimit)/numTicks; //x axis tick step
			double c = firstTick-step;
			int x, yt;

			adj = (w - met.stringWidth(title))/2;
			g.drawString(title,XOFFSET+ adj,fh+5); //draw title
			if(yaxis) {
				double [] r = new double[3]; //r[0] is the step size
				r=graphUtilities.intlabels(0,Ymax,5);

				for (double yy=0; yy<=Ymax+r[0];yy+=r[0]){
					yt = (int) Math.round(hh - sy2p*yy); //y lable positin
					if (yt<=5) break;
					else y = yt;
					g.drawLine(XOFFSET-6,y,XOFFSET-3,y);
					num = graphUtilities.format(yy,2);
					x = XOFFSET - met.stringWidth(num) - 8;
					g.drawString(num,x,y+fh/2);
					//System.out.println("y="+y+" hh="+hh+" fh="+fh+"Ymax="+Ymax);         //y label string
				}
				g.drawLine(XOFFSET-2,y,XOFFSET-2,hh);
			}
		if (autoSetLimits) {//adjust the x lable decimal # according to the steps

			//System.out.println("step="+step);
			if (step >= 1.0) ndx=0;
			if (step < 1.0) ndx=1;
			if (Math.floor(sd) != sd) ndx=1;
			if (step < .05) ndx=2;
		}

		double tickAdj = sx2p*(firstTick-lowLimit);  //adjust the first tick position to include lowlimit
		float SL;
		num=graphUtilities.format(upLimit,ndx);
		SL = (float) met.stringWidth(num);
		double w = step*sx2p;
		int r = (int) Math.ceil(SL/(w-2));
			for (int i=0; i<= numTicks;i++){
				c+=step;
				if (i%r==0) {
				x = (int) Math.rint(XOFFSET + sx2p*i*step + tickAdj); //x label position
				 num=graphUtilities.format(c,ndx);
				g.drawLine(x,hh,x,hh+4);
				SL = (float) met.stringWidth(num);
				 adj = (int) Math.rint(SL/2);
				g.drawString(num, x-adj,hh+fh+6);  //draw x label string
			}
			}


			drawDist(g);  //draw normal distribution


	}

		public void update(Graphics g) {
			Dimension dim = getSize();
			Image osi=createImage(dim.width,dim.height);
			Graphics osg=osi.getGraphics();
			osg.setColor(getBackground());
			osg.fillRect(0,0,dim.width,dim.height);
			osg.setColor(g.getColor());
			paint(osg);
			g.drawImage(osi,0,0,this);
	}

	public  void paramChange() {
			if (autoSetLimits) {
				lowLimit= mean-4*sd;
				firstTick=lowLimit;
				upLimit=mean+4*sd;
				range = upLimit-lowLimit;
			}
			if (autoRepaint) repaint();
		}

	public void drawDist(Graphics g){
			double density;
			int y,h1;

			h1=hh-1;
			density = getDensity(lowLimit);
			lastY = (int) Math.round(hh - sy2p*density); //lowlimit normal probability
			lastX= 0;
			for (int i = pinc; i<w; i+=pinc){ //pinc set to 1 in intialize
				double x = sp2x * i + lowLimit;
				density = getDensity(x);
				y = (int) Math.round(hh - sy2p*density);
				g.setColor(Color.black);
				g.drawLine(XOFFSET+lastX,lastY,XOFFSET+i,y); //x pixel increase pinc, draw the y probability increase
				lastX=i;
				lastY=y;
				if (shade) {
					if (shadeBetween) {
						if ((x>=lowCutoff) && (x<=upCutoff)) { //shade between cut-off points
							g.setColor(fillColor);
							g.drawLine(XOFFSET+lastX,h1,XOFFSET+lastX,y);
						}
					}
					else {
						if ((x<=lowCutoff) || (x>=upCutoff)) { //shade for two tails
							g.setColor(fillColor);
							g.drawLine(XOFFSET+lastX,h1,XOFFSET+lastX,y);
						}
					}
				}
			}

		}
		/**
		 * get density for p(X<=x) for normal distribution
		 */
		public double getDensity(double x) {
			double z=(x-mean)/sd;
			 return MultConst * Math.exp(-z*z/2);
		}

		public  void setYmax(double ymax) {
			Ymax =  ymax;
			setYmax = true;
		}

		public  double getYmax() {
			return MultConst;
		}

	protected double mean;
	protected double sd;
	protected double lowLimit,firstTick;
	protected double upLimit;
	protected double lowCutoff;
	protected double upCutoff;

	protected boolean shade;
	protected boolean shadeBetween;
	protected boolean yaxis;
	protected boolean autoRepaint=true;
	public boolean autoSetLimits = true, setYmax=false;
	protected Color fillColor;

	protected	final int BOTTOM = 55;
	protected	final int HADJ = 35;
	protected final int XOFFSET = 35;
	protected double MultConst,Ymax,range;

	protected int w,h,pinc,lastX,lastY,numTicks,ndec,hh,ndx;
	protected Font font;

	protected String title;
	protected double sx2p,sp2x,sp2y,sy2p,step=1;

}
