package binomialrvls;
import java.awt.*;
public class normalApprox extends normDist{
	double p;  //binomial distribution parameter p
	int N;      //binomial distribution total n
	private int LP,HP; //binomial distribution low bound LP and upper bound HP, for which the probability is > 0
	double [] binProbs; //binomial probabilities for each discrete interger from 0 to N
	public int getHp() {
		return HP;
	}
	public int getLp() {
		return LP;
	}

	/**
	 * call super's initialize, plus set up a basic binomial distribution fitting for p=.5,n=8,LP=0,HP=8;
	 */
	public void initialize(int _numTicks, double _LL, double _UL, double _mean, double _sd,
	 double _LC, double _UC, boolean _between, String _title,
	  Color _fillColor){
	//System.out.println("calling super");
	  super.initialize(_numTicks, _LL,  _UL,  _mean,  _sd,  _LC,  _UC,  _between,"", Color.lightGray);
	   	/*
	   	_mean = N*p;
	  	_sd= Math.sqrt(N*p*(1-p));
	  	setMean(_mean);
	  	setSd(_sd);
	  	*/
	  	//setFillColor(Color.yellow);

	  	setParams(.5,8);
	  	LP=0;HP=8;
	  	firstTick = 0;
	  	setYaxis(true);
	  	autoSetLimits=false; // super class will not change num dec on X-axis;
	  	ndx=0;
	  }

	/**
	 * setParams sets up mean, standard deviation, binomial probability array, and integer
	 * low bound, upper bound for the binomial distribution
	 */
	public void setParams(double _p, int _N) {
		//System.out.println("setting parameters ");
		if (N < 0 || p<0 || p>1) return;
		//System.out.println("N= " + N + " p= " + p);
		N=_N;
		p=_p;

		double meanx=N*p;
		setMean(meanx);
		double q=1-p;
		double sdx = Math.sqrt(N*p*q);
		autoRepaint=false;
		setSd(sdx);
		binProbs = new double[N+1];
		Ymax= 1/(Math.sqrt(2*Math.PI)*sdx) + .1;
		for (int i=0; i<=N; i++) {
			binProbs[i] = b(p,N,i); //set up an array of binomial probability
			Ymax = Math.max(Ymax,binProbs[i]);
			//System.out.println("Ymax="+Ymax+", binProbs["+i+"]="+binProbs[i]);
		}
		hh=getSize().height-BOTTOM + HADJ;
		//System.out.println("Ymax="+Ymax+" hh="+hh+" bottom="+BOTTOM+" HADJ="+HADJ);
		if(hh<=0) {
			LP=0;
			HP=N;
		}
		else {

		sy2p= hh/Ymax;

		int y,y1,LP1,HP1;
		for (LP=-2;LP<=N;LP++) {
			//y = (int) Math.rint(Math.round(sy2p*binProbs[LP]));
			y = (int) Math.rint(Math.round(sy2p*getDensity(LP)));
			if(y >0 ) break;     //the first integer for which the normal probability > 0
		}
		for (LP1=0;LP1<=N;LP1++) {
			y = (int) Math.rint(Math.round(sy2p*binProbs[LP1]));

			if(y >0 ) break; //the first integer for which the binomial probability > 0
		}

		LP = Math.min(LP,LP1);
		for (HP=N+2;HP>=0;HP--) {
			//y = (int) Math.round(sy2p*binProbs[HP]);
			y = (int) Math.round(sy2p*getDensity(HP));//the last integer for which the normal probability > 0
			if(y >0 ) break;
		}


		for (HP1=N;HP1>=0;HP1--) {
			y = (int) Math.round(sy2p*binProbs[HP1]);
			if(y >0 ) break; //the last integer for which the binomial probability > 0
		}

		HP= Math.max(HP,HP1);

	}

		//autoRepaint=true;
		double L= LP-.5;

		double U = HP+.5;
		int R = (int) (U-L);
		setLimits(L,U, R);
		firstTick=LP;
		numTicks = R-1;
		repaint();
	}

	/**
	 * In addition to paint the normal distribution, paint the discrete probablity
	 * of binomial distribution b(n,p) for each integer within the upper and lower limit
	 * range.
	 */
	public void paint(Graphics g) {
		super.paint(g);

		for (int i=LP;i<=HP; i++) {
			if((i>=0) && (i<=N)) {
			double x = (double) i;
			int xp = (int) Math.rint((x - lowLimit)/sp2x+ XOFFSET);
			//double bp = b(p,N,i);
			int y = (int) Math.round(hh - sy2p*binProbs[i]);
			g.setColor(Color.blue);
			g.drawLine(xp,y,xp,hh);
		   }
		}


	}


	/**
	 * calculate the probability of x out of n is true for binomial(n,p) distribution
	 */
	public double b( double p, int n, int x){
			double px = Math.pow(p, (double) x) * Math.pow((1.0-p), (double) (n-x));
			return nper(n,x)* px;
	}


	/**
	 * calculate choosing x from n = n!/x!(n-x)!
	 */
	public double nper(int n, int x){
		int n1 = n+1;
		double r = 1.0;
		int xx =  Math.min(x,n-x);
		for (int i = 1; i< xx+1; i++) {
			r = r *  (n1-i);
			r = r/ i;
		}
		return r;
	}

	/**
	 * cumulative binomial probability from x1 is true to x2 is true
	 */
	public double B(int x1, int x2, double p, int n) {
		double cum=0;
		x2 = Math.min(x2,n);
		x1=Math.max(x1,0);
		for (int i = x1; i<=x2; i++) {
			cum+=b(p,n,i);
		}
		return cum;
	}

	/**
	 * cumulative binomial probability from x1 to x2, if n is predefined
	 */
	public double B(int x1, int x2) {
		return  B(x1,x2,p,N);
	}

	public int getN() {
		return N;
	}

	public double getP(){
		return p;
	}

	public void setN(int N) {
		if (N>0) {
			this.N=N;
			setParams(p,N);
		}
	}
	public void setP(double p) {
		if (p>=0 && p<=1) {
			this.p=p;
			setParams(p,N);
		}
	}


}
